﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace NotObservable
{
    public partial class InternalWindow : Form
    {
        /// <summary>
        /// コンストラクタ
        /// </summary>
        public InternalWindow()
        {
            InitializeComponent();
        }

        /// <summary>
        /// フォーカスアウトイベント
        /// </summary>
        protected virtual void OnBlur(object sender, EventArgs e)
        {
            if (sender == _txtStart)
            {
                OnStartBlur();
            }
            else if (sender == _txtEnd)
            {
                OnEndBlur();
            }
            else if (sender == _txtLength)
            {
                OnLengthBlur();
            }
        }

        /// <summary>
        /// Startのフォーカスアウト時に、Lengthを再計算する。
        /// </summary>
        protected virtual void OnStartBlur()
        {
            if (!isNumber(_txtStart.Text))
            {
                _txtStart.Text = "0";
            }
            CalculateLength();
        }

        /// <summary>
        /// Endのフォーカスアウト時に、Lengthを再計算する。
        /// </summary>
        protected virtual void OnEndBlur()
        {
            if (!isNumber(_txtEnd.Text))
            {
                _txtEnd.Text = "0";
            }
            CalculateLength();
        }

        /// <summary>
        /// Lengthのフォーカスアウト時に、Endを再計算する。
        /// </summary>
        protected virtual void OnLengthBlur()
        {
            if (isNumber(_txtLength.Text))
            {
                _txtLength.Text = "0";
            }
            CalculateEnd();
        }

        /// <summary>
        /// Lengthを再計算して、更新する。
        /// </summary>
        protected virtual void CalculateLength()
        {
            int start = int.Parse(_txtStart.Text);
            int end = int.Parse(_txtEnd.Text);
            int length = end - start;

            _txtLength.Text = length.ToString();
        }

        /// <summary>
        /// Endを再計算して、更新する。
        /// </summary>
        protected virtual void CalculateEnd()
        {
            int start = int.Parse(_txtStart.Text);
            int length = int.Parse(_txtLength.Text);
            int end = start + length;

            _txtEnd.Text = end.ToString();
        }

        /// <summary>
        /// 入力値が数値かどうかを検証する。
        /// </summary>
        /// <param name="input">入力値</param>
        /// <returns>数値ならTrue</returns>
        protected virtual bool isNumber(string input)
        {
            int result = 0;
            return int.TryParse( input, out result );
        }
    }
}
