﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace Observable
{
	/// <summary>
	/// Interval計算画面です。
	/// </summary>
    public partial class InternalWindow : Form, IObserver
	{
		#region フィールド
		/// <summary>
		/// ドメインクラスのインスタンス
		/// </summary>
		protected Interval _subject;
		#endregion

		#region プロパティ
		/// <summary>
		/// ドメインクラスのStart値の取得と設定をします。
		/// </summary>
		public string Start
		{
			get { return this._subject.Start; }
			set { this._subject.Start = value; }
		}

		/// <summary>
		/// ドメインクラスのEnd値の取得と設定をします。
		/// </summary>
		public string End
		{
			get { return this._subject.End; }
			set { this._subject.End = value; }
		}

		/// <summary>
		///ドメインクラスの Length値の取得と設定をします。
		/// </summary>
		public string Length
		{
			get { return this._subject.Length; }
			set { this._subject.Length = value; }
		}
		#endregion

		#region コンストラクタ・生成
		/// <summary>
		/// Interval計算画面です。
        /// </summary>
        public InternalWindow()
        {
            InitializeComponent();

			this._subject = CreateInterval();
			this._subject.RegisterInterest( this );
        }

		/// <summary>
		/// ドメインクラスを生成します。
		/// </summary>
		/// <returns></returns>
		protected virtual Interval CreateInterval()
		{
			return new Interval();
		}
		#endregion

		#region メソッド
		/// <summary>
		/// ドメインクラスが変更されたので、U/Iを更新します。
		/// </summary>
		/// <param name="subject">ドメインクラス</param>
		public void NotifyUpdate( ISubject subject )
		{
			this._txtStart.Text = this._subject.Start;
			this._txtEnd.Text = this._subject.End;
			this._txtLength.Text = this._subject.Length;
		}

		/// <summary>
		/// 入力値が数値かどうかを検証する。
		/// </summary>
		/// <param name="input">入力値</param>
		/// <returns>数値ならTrue</returns>
		protected virtual bool isNumber(string input)
		{
			int result = 0;
			return int.TryParse(input, out result);
		}
		#endregion

		#region イベントハンドラ
		/// <summary>
        /// フォーカスアウトイベント
        /// </summary>
        public virtual void OnBlur(object sender, EventArgs e)
        {
            if (sender == _txtStart)
            {
                OnStartBlur();
            }
            else if (sender == _txtEnd)
            {
                OnEndBlur();
            }
            else if (sender == _txtLength)
            {
                OnLengthBlur();
            }
        }

        /// <summary>
        /// Startのフォーカスアウト時に、Lengthを再計算する。
        /// </summary>
		public virtual void OnStartBlur()
        {
			this.Start = this._txtStart.Text;

			if( !isNumber( this.Start ) )
            {
				this.Start = "0";
            }

			this._subject.CalculateLength();
        }

        /// <summary>
        /// Endのフォーカスアウト時に、Lengthを再計算する。
        /// </summary>
		public virtual void OnEndBlur()
        {
			this.End = this._txtEnd.Text;

			if (!isNumber(this.End))
            {
                this.End = "0";
            }
			this._subject.CalculateLength();
        }

        /// <summary>
        /// Lengthのフォーカスアウト時に、Endを再計算する。
        /// </summary>
		public virtual void OnLengthBlur()
        {
			this.Length = this._txtLength.Text;

			if( !isNumber( this.Length ) )
            {
				this.Length = "0";
            }

			this._subject.CalculateEnd();
        }
		#endregion
	}
}
