﻿using Observable;
using System;
using NUnit.Framework;

namespace ObservableTests
{
    /// <summary>
    ///InternalWindowTest のテスト クラスです。すべての
    ///InternalWindowTest 単体テストをここに含めます
    ///</summary>
	[TestFixture]
	public class InternalWindowTests
	{
		#region ヘルパーメソッド
		/// <summary>
		/// テスト対象のInternalWindowのインスタンスを取得。
		/// </summary>
		/// <returns></returns>
		private InternalWindowExtendForTest GetWindow()
		{
			return new InternalWindowExtendForTest();
		}
		#endregion

		#region コンストラクタ・生成のテスト

		[TestCase()]
		public void InternalWindow_コンストラクタ()
		{
			InternalWindowExtendForTest target = GetWindow();
		}

		#endregion

		#region イベントハンドラのテスト

		[TestCase]
		public void OnBlurTest_Start()
		{
			string expected = "10";

			InternalWindowExtendForTest target = GetWindow();
			target.Start = expected;
			target.OnStartBlur();

			Assert.AreEqual( expected, target.Start );
		}

		[TestCase]
		public void OnBlurTest_End()
		{
			string expected = "20";

			InternalWindowExtendForTest target = GetWindow();
			target.End = expected;
			target.OnEndBlur();

			Assert.AreEqual( expected, target.End );
		}

		[TestCase]
		public void OnBlurTest_Length()
		{
			string expected = "30";

			InternalWindowExtendForTest target = GetWindow();
			target.Length = expected;
			target.OnLengthBlur();

			Assert.AreEqual( expected, target.Length );
		}

		#endregion

		#region オブザーバーのテスト

		[TestCase]
		public void ObserveTest_Start()
		{
			string expected = "10";

			InternalWindowExtendForTest view = GetWindow();
			Interval interval = view.GetDomainObject();
			interval.Start = expected;

			Assert.AreEqual( expected, view.Start );
		}

		[TestCase]
		public void ObserveTest_End()
		{
			string expected = "20";

			InternalWindowExtendForTest view = GetWindow();
			Interval interval = view.GetDomainObject();
			interval.End = expected;

			Assert.AreEqual( expected, view.End );
		}

		[TestCase]
		public void ObserveTest_Length()
		{
			string expected = "30";

			InternalWindowExtendForTest view = GetWindow();
			Interval interval = view.GetDomainObject();
			interval.Length = expected;

			Assert.AreEqual( expected, view.Length );
		}

		#endregion

		#region ドメインクラスのテスト

		[TestCase]
		public void SubjectTest_Start()
		{
			string expected = "10";

			InternalWindowExtendForTest view = GetWindow();
			Interval interval = view.GetDomainObject();
			view.Start = expected;

			Assert.AreEqual( expected, interval.Start );
		}

		[TestCase]
		public void SubjectTest_End()
		{
			string expected = "10";

			InternalWindowExtendForTest view = GetWindow();
			Interval interval = view.GetDomainObject();
			view.End = expected;

			Assert.AreEqual( expected, interval.End );
		}

		[TestCase]
		public void SubjectTest_Length()
		{
			string expected = "10";

			InternalWindowExtendForTest view = GetWindow();
			Interval interval = view.GetDomainObject();
			view.Length = expected;

			Assert.AreEqual( expected, interval.Length );
		}

		#endregion
	}
}
