﻿using Observable;
using System;
using NUnit.Framework;
using Moq;

namespace ObservableTests
{
	[TestFixture]
	public class IntervalTests
	{
		#region ヘルパーメソッド
		/// <summary>
		/// オブザーバーのモックのインスタンスを生成します。
		/// </summary>
		/// <returns>オブザーバーのモックのインスタンス</returns>
		private static IObserver GetObserver()
		{
			Mock<IObserver> mock = new Mock<IObserver>();
			IObserver observer = mock.Object;
			return observer;
		}

		/// <summary>
		/// End値に関するテストを実行します。
		/// </summary>
		/// <param name="start">start値</param>
		/// <param name="length">length値</param>
		private void CheckEnd(int start, int length)
		{
			Interval target = new Interval();

			int end = start + length;
			string expected = end.ToString();

			target.Start = start.ToString();
			target.Length = length.ToString();
			target.End = "0";
			target.CalculateEnd();

			Assert.AreEqual(expected, target.End);
		}

		/// <summary>
		/// Length値に関するテストを実行します。
		/// </summary>
		/// <param name="start">start値</param>
		/// <param name="end">end値</param>
		private void CheckLength(int start, int end)
		{
			Interval target = new Interval();
			int length = end - start;
			string expected = length.ToString();

			target.Start = start.ToString();
			target.End = end.ToString();
			target.Length = "0";
			target.CalculateLength();
			Assert.AreEqual(expected, target.Length);
		}
		#endregion

		#region コンストラクタ・生成のテスト
		[TestCase]
		public void Constractor_Test()
		{
			Interval interval = new Interval();
		}
		#endregion

		#region オブザーバーパターンのテスト
		[TestCase]
		public void RegisterInterest_Test()
		{
			IObserver observer = GetObserver();

			Interval interval = new Interval();
			interval.RegisterInterest( observer );

			Assert.AreEqual( true, interval.IsInterest( observer ) );
		}
		#endregion

		#region データ変更のテスト
		[TestCase]
		public void Start_Test()
		{
			IObserver observer = GetObserver();

			string expected = "10";
			Interval interval = new Interval();
			interval.RegisterInterest( observer );
			interval.Start = expected;

			Assert.AreEqual( expected, interval.Start );
		}

		[TestCase]
		public void End_Test()
		{
			IObserver observer = GetObserver();

			string expected = "10";
			Interval interval = new Interval();
			interval.RegisterInterest( observer );
			interval.End = expected;

			Assert.AreEqual( expected, interval.End );
		}

		[TestCase]
		public void Length_Test()
		{
			IObserver observer = GetObserver();

			string expected = "10";
			Interval interval = new Interval();
			interval.RegisterInterest( observer );
			interval.Length = expected;

			Assert.AreEqual( expected, interval.Length );
		}
		#endregion

		#region ロジックのテスト
		/// <summary>
		/// CalculateLength のテスト
		///</summary>
		[TestCase()]
		public void CalculateLengthTest_プラス()
		{
			CheckLength(1, 3);
		}

		/// <summary>
		///CalculateLength のテスト
		///</summary>
		[TestCase()]
		public void CalculateLengthTest_マイナス()
		{
			CheckLength(3, 1);
		}

		/// <summary>
		///CalculateLength のテスト
		///</summary>
		[TestCase()]
		public void CalculateLengthTest_ゼロ()
		{
			CheckLength(3, 3);
		}

		/// <summary>
		///CalculateLength のテスト
		///</summary>
		[TestCase()]
		public void CalculateLengthTest_異常系()
		{
			Interval target = new Interval();
			target.Start = "a";
			target.End = "b";
			target.Length = "c";
			target.CalculateLength();
			Assert.AreEqual( "0", target.Length );
		}
		/// <summary>
		///CalculateEnd のテスト
		///</summary>
		[TestCase()]
		public void CalculateEndTest_プラス()
		{
			CheckEnd(3, 1);
		}

		/// <summary>
		///CalculateEnd のテスト
		///</summary>
		[TestCase()]
		public void CalculateEndTest_マイナス()
		{
			CheckEnd(3, -4);
		}

		/// <summary>
		///CalculateEnd のテスト
		///</summary>
		[TestCase()]
		public void CalculateEndTest_ゼロ()
		{
			CheckEnd(3, 0);
		}

		/// <summary>
		///CalculateEnd のテスト
		///</summary>
		[TestCase()]
		public void CalculateEndTest_異常系()
		{
			Interval target = new Interval();
			target.Start = "a";
			target.Length = "b";
			target.End = "c";
			target.CalculateEnd();

			Assert.AreEqual( "0", target.End );
		}
		#endregion
	}
}
